// ============================================================================
// Hamster, a free news- and mailserver for personal, family and workgroup use.
// Copyright (c) 1999, Juergen Haible.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
// ============================================================================

unit HConfigAutomatic;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Menus, Clipbrd,
  Mask, cStdForm, cActions, ImgList, CheckLst, ToolEdit;

Function DialogAutomatics: Boolean;

type
  PActionInfo = ^TActionInfo;
  TActionInfo = Record
     Typ: TActionType;
     Key, Script, Exec, Pars, Info: String;
     Wait, ExeStartTyp: Integer;
  end;
  TActiveEntry = Record Node: TTreeNode; Info: PActionInfo end;
  TfrmConfigAutomatics = class(THForm)
    pg: TPageControl;
    tsGeneral: TTabSheet;
    tsActions: TTabSheet;
    OKBtn: TButton;
    CancelBtn: TButton;
    butHelp: TButton;
    chkPurgeDaily: TCheckBox;
    tvActions: TTreeView;
    labChooseAction: TLabel;
    labActionSettings: TLabel;
    frEditAction: TPanel;
    chkWaitProgram: TCheckBox;
    txtPars: TEdit;
    Label3: TLabel;
    Label2: TLabel;
    chkWaitScript: TCheckBox;
    cbScript: TComboBox;
    Label1: TLabel;
    ImageList1: TImageList;
    Label4: TLabel;
    chkCreateStatistics: TCheckBox;
    Label5: TLabel;
    chkPurgeNewsScoreFile: TCheckBox;
    chkCheckObsoleteDirs: TCheckBox;
    chkStartNNTP: TCheckBox;
    chkStartPOP3: TCheckBox;
    chkStartSMTP: TCheckBox;
    labActionInfo: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    chkStartIMAP: TCheckBox;
    chkStartReCo: TCheckBox;
    Label6: TLabel;
    chkOLEServer: TCheckBox;
    tsProcess: TTabSheet;
    gbMailIn: TGroupBox;
    FrMailIn: TCheckListBox;
    gbMailOut: TGroupBox;
    gbNewsIn: TGroupBox;
    gbNewsOut: TGroupBox;
    FrMailOut: TCheckListBox;
    FrNewsIn: TCheckListBox;
    FrNewsOut: TCheckListBox;
    labStartType: TLabel;
    cbStarttype: TComboBox;
    labNewgroupinfoTitle: TLabel;
    chkPurgeMailScoreFile: TCheckBox;
    labNewgroupinfoFilter: TLabel;
    txtNewgroupinfoFilter: TEdit;
    chkPurgeNewsBase: TCheckBox;
    txtProgram: TFilenameEdit;
    procedure FormCreate(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tvActionsChange(Sender: TObject; Node: TTreeNode);
    procedure OnExitActionInputField(Sender: TObject);
  private
    AktEntry: TActiveEntry;
    procedure LoadSettings;
    procedure SaveSettings;
    procedure SaveActions;
    procedure SaveAction;
    procedure FrProcessLoad;
    procedure FrProcessSave;
  end;

implementation

uses Global, cAccount, Config, uTools, dScripts;

{$R *.DFM}

Function DialogAutomatics: Boolean;
begin
   With TfrmConfigAutomatics.Create(NIL) do try
      Result := ShowModal = mrOK
   finally Free end
end;

procedure TfrmConfigAutomatics.LoadSettings;
begin
   With CfgIni do begin
      chkPurgeDaily.Checked := ReadBool ('Setup', 'purge.daily', Def_Purge_Daily);
      chkCheckObsoleteDirs.Checked := ReadBool ('Setup', 'CheckDirs.daily', true);
      chkStartNNTP.Checked := ReadBool( 'Setup', CFG_LOCAL_AUTOSTART_NNTP, True );
      chkStartPOP3.Checked := ReadBool( 'Setup', CFG_LOCAL_AUTOSTART_POP3, True );
      chkStartSMTP.Checked := ReadBool( 'Setup', CFG_LOCAL_AUTOSTART_SMTP, True );
      chkStartIMAP.Checked := ReadBool( 'Setup', CFG_LOCAL_AUTOSTART_IMAP, false ); //IMAP
      chkStartReCo.Checked := ReadBool( 'Setup', CFG_LOCAL_AUTOSTART_ReCo, false );
      chkOLEServer.Checked := ReadBool( 'Setup', 'Startup.OLEServer', Def_OLE_Server);
      chkCreateStatistics.Checked := ReadBool ('Setup', 'CreateStatistics.daily', true);
      chkPurgeNewsScoreFile.Checked := ReadBool ('Setup', 'PurgeNewsScorefile.daily', true);
      chkPurgeMailScoreFile.Checked := ReadBool ('Setup', 'PurgeMailScorefile.daily', true);
      chkPurgeNewsBase.Checked := ReadBool ('Setup', 'PurgeNewsBase.daily', false);
      txtNewgroupinfoFilter.text := ReadString ( 'Setup', 'FilterNewGroupsInfo', Def_FilterNewGroupsInfo);
   end
end;

procedure TfrmConfigAutomatics.SaveSettings;
begin
   With CfgIni do begin
      WriteBool ('Setup', 'purge.daily', chkPurgeDaily.Checked);
      WriteBool ('Setup', 'CheckDirs.daily', chkCheckObsoleteDirs.Checked);
      WriteBool( 'Setup', CFG_LOCAL_AUTOSTART_NNTP, chkStartNNTP.Checked );
      WriteBool( 'Setup', CFG_LOCAL_AUTOSTART_POP3, chkStartPOP3.Checked );
      WriteBool( 'Setup', CFG_LOCAL_AUTOSTART_SMTP, chkStartSMTP.Checked );
      WriteBool( 'Setup', CFG_LOCAL_AUTOSTART_IMAP, chkStartIMAP.Checked ); //IMAP
      WriteBool( 'Setup', CFG_LOCAL_AUTOSTART_ReCo, chkStartReCo.Checked );
      WriteBool( 'Setup', 'Startup.OLEServer', chkOLEServer.Checked);
      WriteBool( 'Setup', 'CreateStatistics.daily', chkCreateStatistics.Checked);
      WriteBool ('Setup', 'PurgeNewsScorefile.daily', chkPurgeNewsScoreFile.Checked);
      WriteBool ('Setup', 'PurgeMailScorefile.daily', chkPurgeMailScoreFile.Checked);
      WriteBool ('Setup', 'PurgeNewsBase.daily', chkPurgeNewsBase.Checked);
      WriteString ( 'Setup', 'FilterNewGroupsInfo', txtNewgroupinfoFilter.text);
   end
end;

Var TI: Integer = 0;

procedure TfrmConfigAutomatics.FormCreate(Sender: TObject);
Var Node1, Node2, Node3: TTreeNode; Userlist: TStringList; b: Boolean;

    Procedure Init ( Akt: TTreeNode; Const AktTyp: TActiontype; Const User, Infotext: String );
    begin
       With PActionInfo(Akt.Data)^ do begin
          Typ := AktTyp;
          Key := ActionIniKeys[Typ];
          If User > '' then Key := Key + '.' + User;
          With CfgIni do begin
             Exec := ReadString ('Actions', Key + '.Exec', '');
             Pars := ReadString ('Actions', Key + '.Pars', '');
             Script := ReadString ('Actions', Key + '.Script', '');
             try Wait := ReadInteger ('Actions', Key + '.Wait', 0)
             except Wait := 0 end;
             try ExeStartTyp := ReadInteger ('Actions', Key + '.ExeStartType', 0)
             except ExeStartTyp := 0 end;
          end;
          Info := Infotext;
          b := Exec+Pars+Script>'';
          SetTreeNodeBold (Akt, b);
          If b then Akt.MakeVisible
       end
    end;

    Procedure Add (Const Level: Integer; Const AktTyp: TActiontype; Const bMultiAccounts: Boolean;
                    Const LangKey, LangDef, LangInfoDef: String );
    Var Title, Infotext: String; Akt: TTreeNode; i, UserID: Integer; Info: PActionInfo;
    begin
       Title := Tr('Title.'+LangKey, LangDef);
       If langInfoDef > ''
          then Infotext := Tr('Title.'+LangKey+'.Info', LangInfoDef)
          else Infotext := '';
       Case Level of
          0: Akt := NIL; 1: Akt := Node1; 2: Akt := Node2; 3: Akt := Node3;
          else exit
       end;
       Akt := tvActions.Items.AddChild ( Akt, Title );
       New(Info);
       Akt.Data := Info;
       Init (Akt, AktTyp, '', Infotext);
       Case Level of
          0: Node1 := Akt; 1: Node2 := Akt; 2: Node3 := Akt;
       end;
       If bMultiAccounts then begin
          for i:=0 to Userlist.Count-1 do begin
             UserID := LongInt( Userlist.Objects[i] );
             Akt := tvActions.Items.AddChild ( Node3, TrF('Title.Mailfor', 'for %s',
                CfgAccounts.Value[UserID, ACTP_FULLNAME]
                + ' (' + CfgAccounts.Value[UserID, ACTP_USERNAME] + ')' ));
             New(Info); Akt.Data := Info;
             Init (Akt, AktTyp, CfgAccounts.Value[UserID, ACTP_USERNAME], Infotext);
          end
       end
    end;

Var st: TExeStartType;
begin
   pg.ActivePage := pg.Pages[TI];

   // Prepare Starttype
   With cbStarttype, Items do begin
      Clear;
      For st := Low(TExeStartType) to High(TExeStartType) do begin
         Add ( TrGl(kGlobal, 'Action.ExeStartType.Caption.'+IntToStr(Ord(st)),
            ExeStartCaption[st] ) )
      end
   end;

   // Prepare Treeview
   tvActions.Items.Clear;
   Userlist := TStringList.Create;
   frEditAction.Visible := false;
   try
      CfgAccounts.SnapshotOfIDs( Userlist );
      Add ( 0, atNone, false, OUR_VERINFO, OUR_VERINFO, '' );
      Add ( 1, atStartUp, false, 'OnStartUp', 'Startup',
         'This action is executed, when Hamster-Fr is started and initialized. No parameters are used.' );
      Add ( 1, atShutDown, false, 'OnShutDown', 'Shutdown',
         'This action is executed, when Hamster-Fr is closed, if you don''t choose "wait", Hamster-Fr can '
          + 'be terminated, before the action is done. No parameters are used.' );
      Add ( 0, atNone, false, 'Statusline', 'Statusline', '' );
      Add ( 1, atStatusClick, false, 'OnClick', 'Click',
         'This action is executed, when the user clicks on the statusline, so it is also executed on '
          +'an double-click! No parameters are used.' );
      Add ( 1, atStatusDblClick, false, 'OnDblClick', 'Double-Click',
         'This action is executed, when the user double-clicks on the statusline. If there'' no action '
          + 'defined, Hamster-Fr opens the dialog with the mail/news-directories. No parameters are used.' );
      Add ( 0, atNone, false, 'TrayIcon', 'Tray-Icon', '' );
      Add ( 1, atTrayIconClick, false, 'OnTrayClick', 'Click',
         'This action is executed, when the user clicks on the tray-icon. If there''s no action '
          + 'defined, Hamster-Fr activates his main window. No parameters used.' );
      Add ( 1, atTrayIconDblClick, false, 'OnTrayDblClick', 'Double-Click',
         'This action is executed, when the user double-clicks on the tray-icon. No parameters are used.' );
      Add ( 1, atTrayIconMiddleClick, false, 'OnTrayIconMiddleClick', 'Click with middle mousebutton',
         'This action is executed, when the user clicks with the middle mousebutton on the tray-icon. '
          + 'No parameters used.' );
      Add ( 1, atTrayIconMiddleDblClick, false, 'OnTrayIconMiddleDblClick', 'Double-Click with middle mousebutton',
         'This action is executed, when the user double-clicks with the middle mousebutton on the tray-icon. No parameters are used.' );
      Add ( 0, atNone, false, 'News', 'News/Postings', '' );
      Add ( 1, atNewsPreprocess, false, 'NewsPreProcess', 'Pre-Process',
         'This action is executed first, when any posting is send through hamster-Fr, and before Hamster-Fr analyses, '
          + 'if all needed headers are set, if it is a local posting etc. Third parameter contains the filename.');
      Add ( 1, atNews, false, 'NewsGeneral', 'Process',
         'This action is executed, before the posting is really saved. If a deeper action is '
          +'defined, too, the general action isn''t used. First parameter contains the filename.' );
      Add ( 2, atNewsOut, false, 'OutgoingNews', 'Outgoing News',
         'This action is used for postings to external newsservers. Third parameter contains the filename.' );
      Add ( 2, atNewsLocal, false, 'LocalNews', 'Local Posting',
         'This action is used for postings to internal groups, Hamster-Fr waits always, until the'
          + 'action has executed. First parameter contains the filename.' );
      Add ( 2, atNewsGateway, false , 'Mail2News', 'Mail-to-News-Gateway',
         'This action is used for mails, which are converted to local postings. This conversion '
          + 'is done, if a "postto"-rule in the Mailfilt.hst is successfull. Hamster-Fr waits '
          + 'always, until the action has executed. First parameter contains the FileName.' );
      Add ( 0, atNone, false, 'Mails', 'Mails', '');
      Add ( 1, atMail, false, 'MailGeneral', 'Process',
         'This action is executed for every incoming or outgoing or local mail, save a deeper action '
          + 'is defined, too, then only the special action is used. '
          + 'First parameter contains the filename, second in some cases the user-account.' );
      Add ( 2, atMailHeader, false, 'MailHeader/2', 'Header of incoming mails',
         'This action is executed before the Mailfilt.hst test the mail, the original headers '
          + 'are saved, Hamster-Fr waits for the end of the action, so you can append or modify '
          + 'headers in the action, which are then checked by the Mailfilt.hst. '
          + 'First parameter contains the filename.' );
      Add ( 2, atMailIn, true, 'IncomingMail', 'Incoming Mail',
         'This action is executed for every incoming Mail (POP3-Server), please be '
          + 'carefully, if you change the mail itself not to damage it! '
          + 'First parameter contains the filename, second the user-account.' );
      Add ( 2, atMailInternal, true, 'InternalMail', 'Internal Mail',
         'This action is executed for every internal Mail (Info-Mails from Hamster-Fr itself). '
          + 'First parameter contains the filename.' );
      Add ( 2, atMailLocal, true, 'LocalMail', 'Local Mail',
         'This action is executed for every Mail to an local account. '
          + 'First parameter contains the filename.' );
      Add ( 2, atMailOut, false, 'OutgoingMail', 'Outgoing Mail',
         'This action is executed for every Mail to an external mailadress. '
          + 'First parameter contains the filename.' );
      Add ( 0, atNone, false, 'Mixed', 'Mixed', '' );
      Add ( 1, atGroupDblClick, false, 'GroupDblClick', 'Double-Click on Newsgroup',
         'This action is executed, when the user double-click on a groupname in the '
          + 'news(server)-configuration-dialog. If not defined, Hamster-Fr opens the '
          + 'group-settings. First parameter is the groupname.' );
      tvActions.Perform ( WM_VSCROLL, SB_TOP, 0 );
      tvActions.Perform ( WM_HSCROLL, SB_LEFT, 0 );
   finally
      Userlist.free
   end;

   // Scriptlist
   Reload_Scriptlist;
   cbScript.Items.Assign ( Scriptlist );

   // Settings
   LoadSettings;

   // OZ - Process
   FrProcessLoad;
end;

Procedure TfrmConfigAutomatics.SaveActions;

   Procedure Test (Const Key, Value, Def: String);
   Var s: String;
   begin
      With CfgIni do begin
         s := ReadString('Actions', Key, Def);
         If s <> Value then begin
            If Value = Def then DeleteKey ('Actions',  Key)
                           else WriteString('Actions', Key, Value)
         end
      end
   end;

Var i: Integer; Info: PActionInfo;
begin
   With tvActions do For i := 0 to Items.Count-1 do With Items[i] do begin
      Info := Data;
      With Info^ do begin
         If Typ > atNone then begin
            Test ( Key + '.Exec', Exec, '' );
            Test ( Key + '.Pars', Pars, '' );
            Test ( Key + '.Script', Script, '' );
            If Exec+Pars+Script > '' then begin
               Test ( Key + '.Wait', IntToStr(Wait), '0' )
            end;
            If Exec > '' then begin
               Test ( Key + '.ExeStartType', IntToStr(ExeStartTyp), '0')
            end
         end
      end
   end
end;

procedure TfrmConfigAutomatics.OKBtnClick(Sender: TObject);
begin
{   CheckForMIDFileChanges;}
   SaveSettings;
   SaveActions;
   ReloadActions;
   // OZ - Process
   FrProcessSave;
   ModalResult := mrOK
end;


procedure TfrmConfigAutomatics.FormDestroy(Sender: TObject);
begin
   TI := pg.ActivePage.PageIndex
end;

procedure TfrmConfigAutomatics.tvActionsChange(Sender: TObject;
  Node: TTreeNode);
Var b: Boolean;
begin
   SaveAction;
   AktEntry.Info := NIL;
   If Assigned(Node) then begin
      AktEntry.Node := Node;
      AktEntry.Info := Node.Data
   end;
   b := Assigned(AktEntry.Info) and (AktEntry.Info^.Typ > atNone);
   frEditAction.Visible := b;
   labActionSettings.Visible := b;
   If b then begin
      labActionInfo.caption := AktEntry.Info^.Info;
      txtProgram.Text := AktEntry.Info^.Exec;
      txtPars.Text := AktEntry.Info^.Pars;
      With cbStartType do try ItemIndex := AktEntry.Info^.ExeStartTyp except ItemIndex := 0 end;
      chkWaitProgram.Checked := (AktEntry.Info^.Wait IN[1, 2]) or (AktEntry.Info^.Typ IN WaitAlways);
      chkWaitProgram.enabled := Not (AktEntry.Info^.Typ IN WaitAlways);
      cbScript.Text := AktEntry.Info^.Script;
      chkWaitScript.Checked := (AktEntry.Info^.Wait IN[1, 3]) or (AktEntry.Info^.Typ IN WaitAlways);
      chkWaitScript.enabled := Not (AktEntry.Info^.Typ IN WaitAlways);
   end
end;

procedure TfrmConfigAutomatics.OnExitActionInputField(Sender: TObject);
begin
   SaveAction
end;

procedure TfrmConfigAutomatics.SaveAction;
Var b: Boolean;
begin
   If Assigned(AktEntry.Info) then begin
      With AktEntry.Info^ do begin
         If Typ > atNone then begin
            Exec := Trim(txtProgram.Text);
            Pars := Trim(txtPars.Text);
            Script := Trim(cbScript.Text);
            If chkWaitProgram.Checked and chkWaitScript.Checked then Wait := 1
            else If chkWaitProgram.Checked then Wait := 2
            else If chkWaitScript.Checked then Wait := 3
            else Wait := 0;
            ExeStartTyp := cbStarttype.ItemIndex;
            b := Exec+Pars+Script > ''
         end else begin
            b := false
         end
      end;
      SetTreeNodeBold ( AktEntry.Node, b )
   end
end;

procedure TfrmConfigAutomatics.FrProcessLoad;
Var p: Integer;
begin
      With FrNewsOut do begin
         Items.Clear;
         p := Items.Add(Tr('Body8bit.desc', 'Body: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsOut.Body8bit', False);
         p := Items.Add(Tr('Headers8bit.desc', 'Headers: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsOut.Headers8bit', False);
         p := Items.Add(Tr('Outlook.desc', 'Outlook: Sig correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsOut.Outlook', False);
         p := Items.Add(Tr('BadRe.desc', 'Outlook: Bad ''RE :'' in subject correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsOut.BadRe', False);
         p := Items.Add(Tr('Cleaner.desc', 'Clears extra newlines at end of body'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsOut.Cleaner', False);
         p := Items.Add(Tr('LineCount.desc', 'Adds ''Lines:'' Header'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsOut.LineCount', False);
       end;
      With FrNewsIn do begin
         Items.Clear;
         p := Items.Add(Tr('Body8bit.desc', 'Body: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.Body8bit', False);
         p := Items.Add(Tr('Headers8bit.desc', 'Headers: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.Headers8bit', False);
         p := Items.Add(Tr('Outlook.desc', 'Outlook: Sig correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.Outlook', False);
         p := Items.Add(Tr('BadRe.desc', 'Outlook: Bad ''RE :'' in subject correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.BadRe', False);
         p := Items.Add(Tr('Cleaner.desc', 'Clears extra newlines at end of body'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.Cleaner', False);
         p := Items.Add(Tr('LineCount.desc', 'Adds ''Lines:'' Header'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.LineCount', False);
         p := Items.Add(Tr('Antipub.desc', 'Clears Advert trailing lines'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrNewsIn.Antipub', False);
       end;
      With FrMailOut do begin
         Items.Clear;
         p := Items.Add(Tr('Body8bit.desc', 'Body: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailOut.Body8bit', False);
         p := Items.Add(Tr('Headers8bit.desc', 'Headers: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailOut.Headers8bit', False);
         p := Items.Add(Tr('Outlook.desc', 'Outlook: Sig correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailOut.Outlook', False);
         p := Items.Add(Tr('BadRe.desc', 'Outlook: Bad ''RE :'' in subject correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailOut.BadRe', False);
         p := Items.Add(Tr('Cleaner.desc', 'Clears extra newlines at end of body'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailOut.Cleaner', False);
         p := Items.Add(Tr('LineCount.desc', 'Adds ''Lines:'' Header'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailOut.LineCount', False);
       end;
      With FrMailIn do begin
         Items.Clear;
         p := Items.Add(Tr('Body8bit.desc', 'Body: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailIn.Body8bit', False);
         p := Items.Add(Tr('Headers8bit.desc', 'Headers: QP or Base64 to 8bit'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailIn.Headers8bit', False);
         p := Items.Add(Tr('Outlook.desc', 'Outlook: Sig correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailIn.Outlook', False);
         p := Items.Add(Tr('BadRe.desc', 'Outlook: Bad ''RE :'' in subject correction'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailIn.BadRe', False);
         p := Items.Add(Tr('Cleaner.desc', 'Clears extra newlines at end of body'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailIn.Cleaner', False);
         p := Items.Add(Tr('LineCount.desc', 'Adds ''Lines:'' Header'));
         Checked[p] := CfgIni.ReadBool ('Fr', 'FrMailIn.LineCount', False);
       end;
end;

procedure TfrmConfigAutomatics.FrProcessSave;
begin
   With FrNewsOut do begin
      CfgIni.WriteBool ('Fr', 'FrNewsOut.Body8bit', Checked[0]);
      CfgIni.WriteBool ('Fr', 'FrNewsOut.Headers8bit', Checked[1]);
      CfgIni.WriteBool ('Fr', 'FrNewsOut.Outlook', Checked[2]);
      CfgIni.WriteBool ('Fr', 'FrNewsOut.BadRe', Checked[3]);
      CfgIni.WriteBool ('Fr', 'FrNewsOut.Cleaner', Checked[4]);
      CfgIni.WriteBool ('Fr', 'FrNewsOut.LineCount', Checked[5]);
   end;
   With FrNewsIn do begin
      CfgIni.WriteBool ('Fr', 'FrNewsIn.Body8bit', Checked[0]);
      CfgIni.WriteBool ('Fr', 'FrNewsIn.Headers8bit', Checked[1]);
      CfgIni.WriteBool ('Fr', 'FrNewsIn.Outlook', Checked[2]);
      CfgIni.WriteBool ('Fr', 'FrNewsIn.BadRe', Checked[3]);
      CfgIni.WriteBool ('Fr', 'FrNewsIn.Cleaner', Checked[4]);
      CfgIni.WriteBool ('Fr', 'FrNewsIn.LineCount', Checked[5]);
      CfgIni.WriteBool ('Fr', 'FrNewsIn.Antipub', Checked[6]);
   end;
   With FrMailOut do begin
      CfgIni.WriteBool ('Fr', 'FrMailOut.Body8bit', Checked[0]);
      CfgIni.WriteBool ('Fr', 'FrMailOut.Headers8bit', Checked[1]);
      CfgIni.WriteBool ('Fr', 'FrMailOut.Outlook', Checked[2]);
      CfgIni.WriteBool ('Fr', 'FrMailOut.BadRe', Checked[3]);
      CfgIni.WriteBool ('Fr', 'FrMailOut.Cleaner', Checked[4]);
      CfgIni.WriteBool ('Fr', 'FrMailOut.LineCount', Checked[5]);
   end;
   With FrMailIn do begin
      CfgIni.WriteBool ('Fr', 'FrMailIn.Body8bit', Checked[0]);
      CfgIni.WriteBool ('Fr', 'FrMailIn.Headers8bit', Checked[1]);
      CfgIni.WriteBool ('Fr', 'FrMailIn.Outlook', Checked[2]);
      CfgIni.WriteBool ('Fr', 'FrMailIn.BadRe', Checked[3]);
      CfgIni.WriteBool ('Fr', 'FrMailIn.Cleaner', Checked[4]);
      CfgIni.WriteBool ('Fr', 'FrMailIn.LineCount', Checked[5]);
   end;
   ConfigLoad (ctAll);
end;

end.

