unit cActions;

interface

Uses Windows, Dialogs;

Type
  TActiontype =
      (atNone,
       atStartUp, atShutDown,
       atNewsPreprocess, atNews, atNewsOut, atNewsLocal, atNewsGateway,
       atMailHeader, atMail, atMailIn, atMailInternal, atMailLocal, atMailOut,
       atGroupDblClick,
       atTrayIconClick, atTrayIconDblClick,
       atTrayIconMiddleClick, atTrayIconMiddleDblClick,
       atStatusClick, atStatusDblClick);
Const
  ActionIniKeys: Array[TActiontype] of String
    = ('',
       'startup', 'shutdown',
       'news.preprocess', 'news', 'news.out', 'news.local', 'news.gateway',
       'mail.getheader', 'mail', 'mail.in', 'mail.internal', 'mail.local', 'mail.out',
       'group.doubleclick',
       'trayicon.click', 'trayicon.doubleclick',
       'trayicon.middleclick', 'trayicon.middledoubleclick',
       'status.click', 'status.doubleclick');
  ActionParent: Array[TActiontype] of TActiontype =
      (atNone,
       atNone, atNone,
       atNone, atNone, atNews, atNews, atNews,
       atNone, atNone, atMail, atMail, atMail, atMail,
       atNone,
       atNone, atNone,
       atNone, atNone,
       atNone, atNone);
  WaitAlways: Set of TActionType =
       [atNews, atNewsPreprocess, atNewsLocal, atNewsGateway,
        atMailHeader];

Type
  TWait = (wNone, wAll, wExec, wScript); // 0, 1, 2, 3
  TExeStartType = (stNormal, stHidden);
Const
  ExeStartValue: Array[TExeStartType] of Integer = ( SW_SHOWNORMAL, SW_SHOWMINNOACTIVE );
  ExeStartCaption: Array[TExeStartType] of String = ( 'normal', 'hidden' );
Type
  TActionInfo = Record
     Tested, Used: Boolean;
     Account, Execute, ExecPar, Script: String;
     Wildcard1, Wildcard0: Integer;
     Wait: TWait;
     ExeStartType: TExeStartType;
  end;
  TActions = Class
    private
      Def_ActionsTimeoutInit, Def_ActionsTimeoutExec: DWord;
      Infos: Array[TActionType] of TActionInfo;
      procedure TestInfo(const Typ: TActiontype; Const Acc: String);
      Function ExecFileAndWait(const aCmdLine: String; Const WorkDir: String;
         Const ShowWindow: Integer; Const doWait: Boolean): Boolean;
    public
      constructor Create;
      Procedure ClearInfo;
      Function Exists (const Typ: TActiontype): Boolean;
      Function Exec(const Typ: TActiontype; Const Parameter: String): boolean;
      Function ExecForAcc(const Typ: TActiontype; Const Acc, Parameter: String): boolean;
  end;

Function Actions: TActions;
Procedure ReloadActions;

implementation

Uses SysUtils, Config, ShellAPI, tScript, cLogFile, cStdForm, uTools;

Var L: TActions;

Function Actions: TActions;
begin
   If Not Assigned(L) then L := TActions.Create;
   Result := L
end;

Procedure ReloadActions;
begin
   If Assigned(L) then FreeAndNil(L)
end;

function TActions.ExecFileAndWait(const aCmdLine: String; Const WorkDir: String;
   Const ShowWindow: Integer; Const doWait: Boolean): Boolean;
var
   StartupInfo : TStartupInfo;
   ProcessInfo : TProcessInformation;
   x: DWord;
   PWorkdir: PChar;
begin
   FillChar(StartupInfo, SizeOf(TStartupInfo), 0);
   with StartupInfo do begin
      cb:= SizeOf(TStartupInfo);
      dwFlags:= STARTF_USESHOWWINDOW or STARTF_FORCEONFEEDBACK;
      wShowWindow:= ShowWindow;


   end;
   If WorkDir > ''
      then PWorkDir := PChar(WorkDir)
      else PWorkDir := NIL;
   Result := CreateProcess(nil,PChar(aCmdLine), nil, nil, False,
                           NORMAL_PRIORITY_CLASS, nil, PWorkDir,
                           StartupInfo, ProcessInfo);
   if doWait and Result then begin
      x := Def_ActionsTimeoutInit;
      If x < INFINITE / 1000 then x := x * 1000 else x := INFINITE;
      Case WaitForInputIdle(ProcessInfo.hProcess, x) of
         WAIT_TIMEOUT: begin
            Log ( LOGID_ERROR, TrGlF(kLog, 'Actions.Exec.Init.Timeout',
               'Error: Timeout when waiting for initialization of "%s"',
               aCmdLine ));
            exit
         end;
         WAIT_FAILED: begin
            try
               RaiseLastOSError;
            except
               On E:Exception do Log ( LOGID_ERROR, TrGlF(kLog, 'Actions.Exec.Init.Error',
                   'Error "%s" when waiting for initialization of %s',
                   [E.message, aCmdLine] ) )
            end;
            exit
         end;
      end;
      x := Def_ActionsTimeoutExec;
      If x < INFINITE / 1000 then x := x * 1000 else x := INFINITE;
      If WaitForSingleObject(ProcessInfo.hProcess, x) = WAIT_TIMEOUT then begin
         Log ( LOGID_ERROR, TrGlF(kLog, 'Actions.Exec.Run.Timeout',
            'Error: Timeout when waiting for termination of %s',
            aCmdLine ) );
         exit
      end
   end
end;

{ TActions }

Const Abschn = 'Actions';

procedure TActions.ClearInfo;
Var i: TActiontype;
begin
   Def_ActionsTimeoutInit := 60;
   Def_ActionsTimeoutExec := 180;
   With CfgIni do begin
      IRead( Abschn, 'Timeout.ExecInit', Def_ActionsTimeoutInit );
      IRead( Abschn, 'Timeout.ExecRun', Def_ActionsTimeoutExec )
   end;
   For i := Low(TActiontype) to High(i) do Infos[i].Tested := false
end;

constructor TActions.Create;
begin
  ClearInfo
end;

procedure TActions.TestInfo(const Typ: TActiontype; Const Acc: String);
Var Key: String;
begin
   With Infos[Typ] do begin
      If Tested and (Acc = Account) then Exit;
      Tested := false;
      Account := '';
      Key := ActionIniKeys[Typ]+'.';
      If Acc > '' then Key := Key + Acc + '.';
      With CfgIni do begin
         Execute := ReadString( Abschn, Key+'exec', '' );
         ExecPar := ReadString( Abschn, Key+'pars', '' );
         Wildcard0 := 0;
         Wildcard1 := 0;
         Script := ReadString( Abschn, Key+'script', '' );
         try Wait := TWait(ReadInteger( Abschn, Key+'wait', 0 ))
         except Wait := wNone end;
         try ExeStartType := TExeStartType(ReadInteger( Abschn, Key+'exestarttype', 0 ))
         except ExeStartType := stNormal end;
      end;
      Used := Execute + Script > '';
      If (Acc > '') and (Not Used) and (ActionParent[Typ] > atNone) then begin
         TestInfo (ActionParent[Typ], Acc);
         If Infos[ActionParent[Typ]].Used
            then Infos[Typ] := Infos[ActionParent[Typ]]
      end;
      If (Acc > '') and (Not Used) then TestInfo (Typ, '');
      If Used then begin
         If (ExecPar > '') and (Wildcard1 = 0) then begin
            Wildcard1 := pos('%1', ExecPar );
            If Wildcard1 > 0 then Delete (ExecPar, Wildcard1, 2)
         end;
         If (ExecPar > '') and (Wildcard0 = 0) then begin
            Wildcard0 := pos('%0', ExecPar );
            If Wildcard0 > 0 then Delete (ExecPar, Wildcard0, 2)
         end;
      end else
      If ActionParent[Typ] > atNone then begin
         TestInfo (ActionParent[Typ], Acc);
         If Infos[ActionParent[Typ]].Used
            then Infos[Typ] := Infos[ActionParent[Typ]]
      end;
      Account := Acc;
      Tested := true
   end
end;

Function TActions.Exec(const Typ: TActiontype; Const Parameter: String): boolean;
begin
   Result := ExecForAcc ( Typ, '', Parameter )
end;

function TActions.ExecForAcc(const Typ: TActiontype; const Acc,
  Parameter: String): boolean;
Var AktPar, WorkDir: String;
    W: TWait; p2: Integer;
begin
   TestInfo (Typ, Acc);
   With Infos[Typ] do begin
      Result := Used;
      If Not Result then Exit;
      If Typ IN WaitAlways then W := wAll else W := Wait;
      If Execute > '' then begin
         AktPar := ExecPar;
         WorkDir := ExtractFileDir(Execute);
         If WildCard1 > 0 then Insert ( Parameter, AktPar, WildCard1 );
         If WildCard0 > 0 then begin
            p2 := WildCard0;
            If (WildCard1 > 0) and (WildCard1 < p2) then p2 := p2 + Length(Parameter);
            Insert ( LowerCase(ActionIniKeys[Typ]), AktPar, p2 )
         end;
         If W IN [wAll, wExec] then begin
            Log (LOGID_INFO, TrGlF (kLog, 'Action.ExecWait', 'Action-Type %s - Execute and wait: %s',
               [ActionIniKeys[Typ], Execute+' '+AktPar] ) );
            ExecFileAndWait('"'+Execute+'" '+AktPar, Workdir, ExeStartValue[ExeStartType], true)
         end else begin
            Log (LOGID_INFO, TrGlF (kLog, 'Action.ExecAsync', 'Action-Type %s - Execute: %s',
               [ActionIniKeys[Typ], Execute+' '+AktPar] ) );
            ShellExecute( 0, 'open', PChar(Execute), PChar(AktPar),
               PChar(WorkDir), ExeStartValue[ExeStartType] )
         end
      end;
      If Script > '' then begin
         AktPar := LowerCase(ActionIniKeys[Typ]) + #13#10 + Parameter;
         If Acc > '' then AktPar := AktPar + #13#10 + Acc;
         If W IN [wAll, wScript] then begin
            Log (LOGID_INFO, TrGlF (kLog, 'Action.ScriptWait2', 'Action-Type %s - run script "%s", Wait=%s',
               [ActionIniKeys[Typ], Script, '1'] ) );
            StartNewScript ( Script, AktPar, true)
         end else begin
            Log (LOGID_INFO, TrGlF (kLog, 'Action.ScriptWait2', 'Action-Type %s - run script "%s", Wait=%s',
               [ActionIniKeys[Typ], Script, '0'] ) );
            StartNewScript ( Script, AktPar, false)
         end
      end
   end
end;

function TActions.Exists(const Typ: TActiontype): Boolean;
begin
   TestInfo (Typ, '');
   Result := Infos[Typ].Used
end;


initialization
   L := NIL;
finalization
   If Assigned(L) then FreeAndNil(L)
end.
