// ============================================================================
// Hamster, a free news- and mailserver for personal, family and workgroup use.
// Copyright (c) 1999, Juergen Haible.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
// ============================================================================

unit dAbout; // Dialog: About ...

interface

uses SysUtils, ExtCtrls, Controls, StdCtrls, ComCtrls, Forms, Classes, uTools,
  Graphics, cStdForm, Windows, ShellAPI;

type
  TDlgAbout = class(THForm)
    OKButton: TButton;
    pg: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    mLicence: TMemo;
    mCredits: TMemo;
    Panel1: TPanel;
    ProgramIcon: TImage;
    Bevel2: TBevel;
    Version: TLabel;
    Maintainer: TLabel;
    Download: TLabel;
    emVersion: TEdit;
    emMaintainer: TEdit;
    emProductName: TEdit;
    ProductName: TLabel;
    TabSheet4: TTabSheet;
    mAuthors: TMemo;
    TabSheet5: TTabSheet;
    Button1: TButton;
    mChanges: TRichEdit;
    DevImage: TImage;
    Label1: TLabel;
    Thanks: TTabSheet;
    mThanks: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure OKButtonDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure OKButtonDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure Label1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

Var
  DlgAbout: TDlgAbout = nil;

// ----------------------------------------------------------------------------

implementation

Uses Config, global;

{$R *.DFM}

Const

  InfoAuthors =
   '"Hamster-Classic" is based on "Hamster Vr. 1.3.21.0", which was released as ' +
   'Open Source in July 2000. The addition "-Classic" was used to distinguish it ' +
   'from other successors of Hamster.^m^m' +
   'The authors of "Hamster-Classic" are:^m' +
   '  Juergen Haible, <juergen.haible@t-online.de>^m'+
   '  Thomas G. Liesner, <tgl@gmx.de>^m'+
   '  Joern Weber <listen@joernweber.de>^m'+
   '  Heiko Studt <useHamster@myokay.net>^m'+
   '  Philipp Kern <mail@philkern.de>^m'+
   '  Martin Germann <listen@ximera.inka.de>^m' +
   '  Wolfgang Jaeth <jawo.ml.hamster@freenet.de>^m' +
   '  and many more...^m^m'+
   'See file Changes.txt for more details on who did what.^m^m' +
   'In the first French versions, the ''0'' had been replaced by the version number.^m' +
   'Starting with 1.3.21.7 numbering became 1.7.x.x.^m^m' +
   'Versions 1.x.x.x are based on Hamster-Classic 1.3.21.0.^m' +
   'Versions 2.x.x.x are based on Hamster-Classic 1.3.22.0 and later ' +
   'and are language-file compliants.^m^m' +
   'The developers of "Hamster-Fr" are :^m' +
   '  Gildas Le Gouic <glegouic@hamster-fr.org>^m' +
   '  Olivier Zolli <ozolli@hamster-fr.org>^m' +
   '  Guy Capra <contact@alomphega.com>^m' +
   '  Christophe Esperado <ch.esperado@libertysurf.fr>^m' +
   '  Thomas Lemoine <lemoine@nerim.net>^m' +
   '  Laurent Sguin <laurent@seguin.com>^m';

  InfoLicence_a =
   '==============================================================^m' +
   'Hamster, a free news- and mailserver for personal, family and workgroup use. ' +
   'Copyright (c) 1999, Juergen Haible (<juergen.haible@t-online.de>).^m^m' +
   'Permission is hereby granted, free of charge, to any person obtaining a copy of this ' +
   'software and associated documentation files (the "Software"), to deal in the Software ' +
   'without restriction, including without limitation the rights to use, copy, modify, merge, ' +
   'publish, distribute, sublicense, and/or sell copies of the Software, and to permit ' +
   'persons to whom the Software is furnished to do so, subject to the following ' +
   'conditions:^m^m' +
   'The above copyright notice and this permission notice shall be included in all copies ' +
   'or substantial portions of the Software.^m^m';
  InfoLicence_b =
   'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, ' +
   'EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES ' +
   'OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND ' +
   'NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT ' +
   'HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, ' +
   'WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING ' +
   'FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR ' +
   'OTHER DEALINGS IN THE SOFTWARE.^m' +
   '==============================================================';

  InfoCredits =
   'The "regular expression"-routines used in Hamster use a Win32-port of ' +
   '"Perl-Compatible Regular Expression (PCRE)" written by Philip Hazel ' +
   '(<ph10@cam.ac.uk>), Copyright (c) 1997-2000 by University of Cambridge (see ' +
   'ftp://ftp.csx.cam.ac.uk/pub/software/programming/pcre/ for more details).^m^m' +
   'Although the wrapper-class used for PCRE uses a slightly different approach, parts of ' +
   'it are at least inspired by the concepts used in "uperlre". "uperlre" is a wrapper-class ' +
   'for PCRE written by Luu Tran (<xnews@newsguy.com>), author of ' +
   '"XNews"-newsreader (see http://xnews.newsguy.com/ for more details).^m^m' +
   'Thanks! :-)';

  InfoOpenSSL =
   '^m^mThis product includes software developed by the OpenSSL Project for ' +
   'use in the OpenSSL Toolkit (http://www.openssl.org/). This product ' +
   'includes cryptographic software written by Eric Young (eay@cryptsoft.com). ' +
   'This product includes software written by Tim Hudson (tjh@cryptsoft.com).';

  InfoIndy_a =
   '^m^m==============================================================^m' +
   'Indy Modified BSD License^m^m' +
   'Copyright^m^m' +
   'Portions of this software are Copyright (c) 1993 - 2001, Chad Z. Hower ' +
   '(Kudzu) and the Indy Pit Crew - http://www.nevrona.com/Indy/^m^m' +
   'License^m^m' +
   'Redistribution and use in source and binary forms, with or without ' +
   'modification, are permitted provided that the following conditions are met:^m^m' +
   'Redistributions of source code must retain the above copyright notice, this ' +
   'list of conditions and the following disclaimer.^m^m' +
   'Redistributions in binary form must reproduce the above copyright notice, ' +
   'this list of conditions and the following disclaimer in the documentation, ' +
   'about box and/or other materials provided with the distribution.^m^m' +
   'No personal names or organizations names associated with the Indy project ' +
   'may be used to endorse or promote products derived from this software ' +
   'without specific prior written permission of the specific individual or ' +
   'organization.^m^m';

  InfoIndy_b =
   'THIS SOFTWARE IS PROVIDED BY Chad Z. Hower (Kudzu) and the Indy Pit Crew ' +
   '"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED ' +
   'TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR ' +
   'PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE ' +
   'LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR ' +
   'CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF ' +
   'SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS ' +
   'INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN ' +
   'CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ' +
   'ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE ' +
   'POSSIBILITY OF SUCH DAMAGE.^m' +
   '==============================================================';

  InfoThanks =
   'Many thanks to each people contributing to make Hamster-Fr what it is now, especially:^m^m' +
   '  The development team ;-)^m' +
   '  Claire Brione^m' +
   '  Manitou^m' +
   '  Sophie Launay^m' +
   '  Frdric Bzis^m' +
   '  Patrick Lamaizire^m' +
   '  Jrme Fivre^m' +
   '  Herv Guilbert^m' +
   '  Florent Faessel^m' +
   '  Sylvie Pierrard^m' +
   '  Pascale^m' +
   '  Gilles Ronsin^m' +
   '  Max Puech^m' +
   '  Sbastien Morin^m' +
   '  Gils Gayraud^m' +
   '  Xavier^m' +
   '  Jean-Christophe Brenier^m' +
   '  Stphane Catteau^m' +
   '  Michel Guillou^m' +
   '  Stphane Marchau^m' +
   '  Mat^m' +
   '  Vincent Hiribarren^m' +
   '  Jeff^m' +
   '  Jos S^m' +
   '  Teke Akhal^m' +
   '  Jean Rousseau^m' +
   '  Alain Banyuls^m' +
   '  Philippe Vigeral^m' +
   '  MC2^m' +
   '  Gilles Goullet^m' +
   '  Olivier Hoarau^m' +
   '  Jean-Pierre Parisy^m' +
   '  Gastibelza^m' +
   '  Franoise Marre-Fournier^m' +
   '  Le sale gosse^m' +
   '  Aymerick^m' +
   '  Michel Claveau^m' +
   '  Steven^m';

procedure TDlgAbout.FormCreate(Sender: TObject);
Var Bez: String;
begin
   DevImage.Hide;
   pg.ActivePage := pg.Pages[0];
   emProductName.Text := GetMyStringFileInfo( 'ProductName', OUR_VERINFO);
   emMaintainer.Text  := GetMyStringFileInfo( 'Maintainer',  '(not set)' );
   emVersion.Text     := GetMyVersionInfo  + ' ' + GetMyStringFileInfo('Comments','');
   mAuthors.Text := Tr('AuthorsV2', InfoAuthors);
   mLicence.Text := Tr('LicenceV1/a', InfoLicence_a) + Tr('LicenceV1/b', InfoLicence_b) +
      Tr('LicenceIndy/a', InfoIndy_a) + Tr('LicenceIndy/b', InfoIndy_b);
   mThanks.Text := Tr('Thanks', InfoThanks);

   if SSLReady then mLicence.Text := mLicence.Text + Tr('OpenSSL', InfoOpenSSL); {MG}{SSL}

   mCredits.Text := Tr('CreditsV2', InfoCredits);

   Bez := PATH_BASE + TabSheet5.caption;
   If FileExists(Bez) then mChanges.Lines.LoadFromFile(Bez);

   With emProductName.Font do begin Style := [fsBold]; Size := Size + 4; end;
   With emMaintainer.Font do Style := [fsBold];
   With emVersion.Font do Style := [fsBold];

   Label1.Caption := GetMyStringFileInfo( 'Download',  '(not set)' );
   With Label1.Font do Style := [fsBold,fsUnderline];
end;

procedure TDlgAbout.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     Release;
     DlgAbout := nil;
end;

// ----------------------------------------------------------------------------

procedure TDlgAbout.Button1Click(Sender: TObject);
begin
   Help
end;

procedure TDlgAbout.OKButtonDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source is TImage;
end;

procedure TDlgAbout.OKButtonDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  if (Sender is TButton) and (Source is TImage) then
  begin
    with Sender as TButton do
    begin
      DevImage.Show;
    end;
  end;
end;

procedure TDlgAbout.Label1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 inherited MouseDown(Button, Shift, X, Y);
 ShellExecute(ValidParentForm(Self).Handle,'open',PChar(Label1.Caption),
              nil,nil,SW_SHOWNORMAL);

end;

end.

