{*******************************************************}
{                                                       }
{       HTML Help API Interface Unit                    }
{                                                       }
{       Copyright (c) 1999 The Helpware Group           }
{                                                       }
{*******************************************************}

{
========================================================
  hh.pas
  Version: 1.6
  HTML Help API Unit

  Modified by OZ for Hamster-Fr
========================================================
}

unit hh;

interface

uses
  Windows, SysUtils, Registry;

var
  //0 if hhctrl.ocx could not be loaded
  HHCtrlHandle: THandle = 0;

const
  hhctrlLib  = 'hhctrl.ocx';

{exports}
  function GetPathToHHCtrlOCX: string;
  procedure LoadHtmlHelp;
  procedure UnloadHtmlHelp;


{ Externals from HHCTRL.OCX }

var                            //functions are invalid if HHCtrlHandle = 0
  HtmlHelpA: function(hwndCaller: HWND; pszFile: PAnsiChar;
    uCommand: UInt; dwData: DWORD): HWND; stdcall;

  HtmlHelpW: function(hwndCaller: HWND; pszFile: PWideChar;
    uCommand: UInt; dwData: DWORD): HWND; stdcall;

  HtmlHelp: function(hwndCaller: HWND; pszFile: PChar;
    uCommand: UInt; dwData: DWORD): HWND; stdcall;

  { Commands to pass to HtmlHelp() }

const
  HH_DISPLAY_TOPIC        = $0000;  {**}
  HH_HELP_CONTEXT         = $000F;  {**}// display mapped numeric value in dwData
  HH_CLOSE_ALL            = $0012;  // close all windows opened directly or indirectly by the caller

implementation

uses cStdForm, cLogFile;

{Return Windows System Dir - with no trailing slash}
function GetWinSysDir: String;
var path: array[0..260] of Char;
begin
  GetSystemDirectory(path, SizeOf(path));
  result := path;
  if result[length(result)] = '\' then
    SetLength(result, length(result)-1);
end;

const hhPathRegKey = 'CLSID\{adb880a6-d8ff-11cf-9377-00aa003b7a11}\InprocServer32';

{ Returns full path to hhctrl.ocx.
  Returns empty string if file or registry entry not found.
  Note: hhctrl.ocx may not be in the path. Consider the case where
        the ocx has been downloaded to the windows ocx cache via the net.
        So.. best to get the path from the registry.
}
function GetPathToHHCtrlOCX: string;
var Reg: TRegistry;
begin
  result := '';  //default return
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_CLASSES_ROOT;
  if Reg.OpenKeyReadOnly(hhPathRegKey) then  //safer call under NT
  begin
    result := Reg.ReadString('');  //default value
    Reg.CloseKey;
    if (result <> '') and (not FileExists(result)) then  //final check - file must exist
      result := '';
  end;
  Reg.Free;
end;


{setup HTML Help API function interface
 sets HHCtrlHandle = 0 if API function not available }
procedure LoadHtmlHelp;
var OcxPath: string;
begin
  if HHCtrlHandle = 0 then
  begin
    OcxPath := GetPathToHHCtrlOCX;
    if (OcxPath <> '') and FileExists(OcxPath) then
    begin
      HHCtrlHandle := LoadLibrary(PChar(OcxPath));
      if HHCtrlHandle <> 0 then
      begin
        @HtmlHelpA := GetProcAddress(HHCtrlHandle, 'HtmlHelpA');
        @HtmlHelpW := GetProcAddress(HHCtrlHandle, 'HtmlHelpW');
        @HtmlHelp := GetProcAddress(HHCtrlHandle, 'HtmlHelpA');
      end;
    end;
  end;
end;

procedure UnloadHtmlHelp;
begin
  if HHCtrlHandle <> 0 then
  begin
    FreeLibrary(HHCtrlHandle);
    @HtmlHelpA := nil;
    @HtmlHelpW := nil;
    @HtmlHelp := nil;
    HHCtrlHandle := 0;
  end;
end;

initialization
  LoadHtmlHelp;
  if HHCtrlHandle = 0 then
    log(LOGID_WARN, TrGl(kLog,'HHNotInstalled', 'HTML Help is not installed on this PC.'));

finalization
  UnloadHtmlHelp;
end.



