// ============================================================================
// Hamster, a free news- and mailserver for personal, family and workgroup use.
// Copyright (c) 1999, Juergen Haible.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
// ============================================================================

unit tScript; // Threads which runs scripts

// ----------------------------------------------------------------------------
// Contains the thread which executes scripts and the Hamster-related
// enhancements of the script-engine.
// ----------------------------------------------------------------------------

{$INCLUDE Compiler.inc}

interface

{JW} {SSL CMD}
uses Windows, SysUtils, Forms, Classes,
     {$IFDEF D6GE} Variants, {$ENDIF}
     tBase, cHscHamster, cHSCEngine, cSyncObjects;
{JW}

type
  TScriptmode = (smFile, smLine);
  TThreadExecuteScript = class( TBaseThread )
    private
      FScriptMode: TScriptmode;
      FScriptFilename: String;
      FScriptCommands: TStringlist;
      FParameterList: String; // JAWO Params 4.10.2000 (new variable)
      FOutputBuffer: TThreadStringList;
    protected
      procedure Execute; override;
    public
      property OutputBuffer: TThreadStringList read FOutputBuffer;
      constructor Create( const AScriptFile, AParamsText: String;
                          const DoBufferOutput: Boolean = False;
                          const DoFreeOnTerminate: Boolean = True ); overload;
      constructor Create( AScript: TStrings;
                          Const AParameterList: String ); overload;
      destructor Destroy; override;
  end;

  function StartNewScript ( ScriptFile, ParameterList: String;
                            WaitForEnd : Boolean ): Integer; overload;
  function StartNewScript( const ScriptFile, ParamsText: String;
                           const WaitForEnd : Boolean;
                           out   ScriptThread: TThread;
                           const DoBufferOutput: Boolean = False;
                           const DoFreeOnTerminate: Boolean = True ): Integer; overload;
  function StartScriptLines ( AScript: TStrings ): Integer;

implementation

uses Global, Config, cPasswordFile, uTools, cArticle, cArtFiles, cFiltersNews,
     tTransfer, tMaintenance, Controls, Dialogs, XApp, cStdForm, Main,
     cMailRouter, cLogfile;
(*
function TThreadExecuteScript.Engine_Func (ScriptLine: THscScriptLine ): Variant;
var  i, j, LfdServer, LfdThread, TimeOut: Integer;
     k: Integer;
     s, t, SrvList, s1,s2,s3: String;
     Identifier, ConnectionID, Username,  Password: String;
     Server, Port, User, Pass, DestUser, FiltSect, FromSel, LeaveOnServer: String;
     Art: TArticle;
     ScoreFile: TFiltersNews;
     XOverRec : TXOverRec;
     {JW} {SSL}
     OK,UseSSL: Boolean;
     SSLPort,SSLOption: String;
     {JW}

     {JW} {purge event}
      Function CheckPurge (Const bShowWarning: Boolean): boolean;
      begin
          Result := WAIT_OBJECT_0=WaitForSingleObject(EventPurge,0);
          If Result and bShowWarning
             then Log(LogID_Warn, TrGl(kLog, 'Script.Abort.PurgeDatabase',
                  'critical state detected: Hamster purges database, hs2 command aborted'))
      end;
      {JW}

begin
   Result := NULL;
   with ScriptLine do begin
{JW} {Tasks}
      end else if FuncIs( 'settasklimiter', 1, 1) then begin
         result:=Def_MaxTasks;
         Def_MaxTasks:=ParI(0,0);
      end else if FuncIs( 'gettasksactive', 0, 0) then begin
         EnterCriticalSection(CS_Counter);
         result:=CntActiveTasks;
         LeaveCriticalSection(CS_Counter);
      end else if FuncIs( 'gettaskswait', 0, 0) then begin
         EnterCriticalSection(CS_Counter);
         result:=CntWaitTasks;
         LeaveCriticalSection(CS_Counter);
      end else if FuncIs( 'gettasksrun', 0, 0) then begin
         EnterCriticalSection(CS_Counter);
         result:=CntActiveTasks-CntWaitTasks;
         LeaveCriticalSection(CS_Counter);
{JW}
{JW} {SSL CMD}
      end else if FuncIs( 'openssltunnel', 4, 5 ) then begin
         Result :=Integer(StartSSLWrapper(ParS(0,''),ParS(1,''),
                          ParS(2,''),ParI(3,0),ParS(4,'')));
      end else if FuncIs( 'closessltunnel', 1, 1 ) then begin
         Result :=Integer(StopSSLWrapper(ParI(0,0)));
{JW}
{JW} {lasterror}
      end else if FuncIs( 'raslasterror',  0, 0 ) then begin
         Result := RasDialer.LastError;
{JW}

      end
   end
end;
-----------------------------------------------------
*)

procedure TThreadExecuteScript.Execute;
var  MyScriptList: TStringList;
     MyScriptName: String;
     HSCResult: THscVariant;
begin
   If FScriptMode = smFile then begin
      TLog( LOGID_SYSTEM, TrGl(kLog, 'Skript.Start', 'Start') )
   end;

   AllShutdownReq := False;
   {JW} {critical event}
   EnterCriticalSection(CS_Event);
   ResetEvent( EVT_STOPSCRIPT );
   LeaveCriticalSection(CS_Event);
   {JW}

   MyScriptList := TStringList.Create;
   try
      If FScriptMode = smFile then begin
         MyScriptName := ExtractFilename( FScriptFilename );
         try
            MyScriptList.LoadFromFile( FScriptFilename );
         except
            TLog( LOGID_SYSTEM, TrGl(kLog, 'Skript.End', 'End' ) );
            TLog( LOGID_ERROR, TrGlF(kLog, 'Skript.ErrorOpening',
                  'Error opening %s', FScriptFilename) );
            exit
         end
      end else begin
         MyScriptName := '';
         MyScriptList.Assign ( FScriptCommands )
      end;

      HSCResult := THscVariant.Create( 0, false );
      with THscEngineHamster.Create( Self, PATH_HSM, Synchronize, FOutputBuffer ) do try
         ExecuteFromList( MyScriptName, MyScriptList, FParameterList, HSCResult );
      finally
         Free;
         HSCResult.Free
      end;

      NewsHistory.SaveToFile;

      if ShutDownReq and RasDialer.IsConnected then begin
         if MessageDlg( 'Hangup?', mtConfirmation, [mbYes,mbNo], 0 )=mrYes then begin
            RasDialer.HangUp
         end
      end;
   finally
      MyScriptList.Free;
      If FScriptMode = smFile then TLog( LOGID_SYSTEM, TrGl(kLog, 'Skript.End', 'End' ) )
   end;
end;

constructor TThreadExecuteScript.Create(
                          const AScriptFile, AParamsText: String;
                          const DoBufferOutput: Boolean = False;
                          const DoFreeOnTerminate: Boolean = True );
begin
     inherited Create( '{script ' + ExtractFilename(AScriptFile)+ '}',
        DoFreeOnTerminate );
     FScriptMode := smFile;
     FScriptFileName := AScriptFile;
     FScriptCommands := NIL;
     FParameterList := AParamsText;
     if DoBufferOutput
        then FOutputBuffer := TThreadStringList.Create( False, dupAccept )
        else FOutputBuffer := nil;
end;

constructor TThreadExecuteScript.Create( AScript: TStrings; Const AParameterList: String );
begin
   inherited Create( '{script}' );
     FScriptMode := smLine;
     FScriptFilename := '';
     FScriptCommands := TStringList.Create;
     FScriptCommands.Assign(AScript);
     FParameterList := AParameterList;
     FOutputBuffer := NIL;
end;

destructor TThreadExecuteScript.Destroy;
begin
  If FScriptMode = smFile
     then MainLog (LOGID_SYSTEM, TrGlF(kLog, 'Script.end', 'Script %s ended.', FScriptFilename) )
     else MainLog (LOGID_SYSTEM, TrGl(kLog, 'Scriptline.end', 'Scriptline(s) executed.') );
  If Assigned(FScriptCommands) then FreeAndNil(FScriptCommands);
  inherited
end;

function StartNewScript( const ScriptFile, ParamsText: String;
                         const WaitForEnd : Boolean;
                         out   ScriptThread: TThread;
                         const DoBufferOutput: Boolean = False;
                         const DoFreeOnTerminate: Boolean = True ): Integer;
var  Filename, Params: String;
begin
   Result := -1;
   ScriptThread := nil;
   AllShutdownReq := False;

   Log( LOGID_SYSTEM, TrGl(kLog, 'Script.Starting', 'Starting script') + ': ' +
        ScriptFile + ', Wait=' + inttostr(Ord(WaitForEnd)));
   if pos (LF, ParamsText)<> 0 then begin
      Log( LOGID_DEBUG, TrGlF(kLog, 'Script.StartingPars',
          'Starting script: Parameters = "%s"',
          [copy (ParamsText, pos (LF, ParamsText)+1, Length(ParamsText))]))
   end else begin
      Log( LOGID_DEBUG,
            TrGl(kLog, 'Script.StartingNoPars', 'Starting script'))
   end;

   FileName := ScriptFile;
   If ExtractFileExt(FileName) = '' then FileName := FileName + '.hsc';
   if (Pos(':', FileName)=0) and (Pos('\', Filename)<>1) then Filename := PATH_HSC + FileName;
   If Not FileExists2( Filename ) then begin
      Log( LOGID_ERROR, TrGlF(kLog, 'Script.Not_Found', 'Script "%s" not found!', Scriptfile) );
      Result := -2;
      exit
   end;
   if (Filename > '') then begin
      Params := ScriptFile;
      If ParamsText > '' then Params := Params + LF + ParamsText;
      if WaitForEnd then begin
         ScriptThread := TThreadExecuteScript.Create(
                         Filename, Params, DoBufferOutput,
                         false );
         ScriptThread.Resume;
         ScriptThread.WaitFor;
         FreeAndNil(ScriptThread)
      end else begin
         ScriptThread := TThreadExecuteScript.Create(
                         Filename, Params, DoBufferOutput,
                         DoFreeOnTerminate );
         ScriptThread.Resume
      end;
      Result := 0;
   end
end;

function StartNewScript ( ScriptFile, ParameterList: String;
                            WaitForEnd : Boolean ): Integer; overload;
var  ScriptThread: TThread;
begin
    Result := StartNewScript( ScriptFile, ParameterList, WaitForEnd,
                              ScriptThread, False, True );
end;

function StartScriptLines ( AScript: TStrings ): Integer;
var  ScriptThread: TThread;
begin
   // Log( LOGID_SYSTEM, TrGl(kLog, 'ScriptLines.Starting', 'Starting script-line(s)'));
   AllShutdownReq := False;
   Result := 0;
   ScriptThread := TThreadExecuteScript.Create( AScript, '');
   ScriptThread.resume;
   if Assigned(ScriptThread) then begin
      // Log( LOGID_INFO, TrGl(kLog, 'ScriptLines.Started', 'Script-line(s) started.' ) );
      Result := 0;
   end else begin
      Log( LOGID_ERROR, TrGl(kLog, 'ScriptLines.Not_Created', 'Cannot create Thread for Scriptline(s)!') )
   end
end;

end.

