unit zDNSfinder;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
   Dialogs, StdCtrls, cStdForm, cLogfile, Config, Global;

Procedure Dialog_DNSFinder;
function GetDNS(Index: Integer): String;
function DNSFinder( const List: TStrings ): Boolean;

const
   MAX_HOSTNAME_LEN    = 128;
   MAX_DOMAIN_NAME_LEN = 128;
   MAX_SCOPE_ID_LEN    = 256;

type
   TfrmDNSfinder = class(THForm)
      butDetect: TButton;
      ButCancel: TButton;
      butOK: TButton;
      Label1: TLabel;
      Label6: TLabel;
      gbDetected: TGroupBox;
      gbConfig: TGroupBox;
      Label4: TLabel;
      ComboBox1: TComboBox;
      Label5: TLabel;
      ComboBox2: TComboBox;
      lbDetected: TListBox;
      procedure FormCreate(Sender: TObject);
      procedure butOKClick(Sender: TObject);
      procedure butDetectClick(Sender: TObject);
   end;

   IP_ADDRESS_STRING = record
      S: array [0..15] of Char;
   end;

   IP_MASK_STRING = IP_ADDRESS_STRING;

   PIP_ADDR_STRING = ^IP_ADDR_STRING;
   IP_ADDR_STRING = record
      Next: PIP_ADDR_STRING;
      IpAddress: IP_ADDRESS_STRING;
      IpMask: IP_MASK_STRING;
      Context: DWORD;
   end;

   PFIXED_INFO = ^FIXED_INFO;
   FIXED_INFO = record
      HostName: array [0..MAX_HOSTNAME_LEN + 3] of Char;
      DomainName: array[0..MAX_DOMAIN_NAME_LEN + 3] of Char;
      CurrentDnsServer: PIP_ADDR_STRING;
      DnsServerList: IP_ADDR_STRING;
      NodeType: UINT;
      ScopeId: array [0..MAX_SCOPE_ID_LEN + 3] of Char;
      EnableRouting: UINT;
      EnableProxy: UINT;
      EnableDns: UINT;
   end;

   TGetNetworkParams = function(
      pFixedInfo: PFIXED_INFO; var pOutBufLen: ULONG ): DWORD; stdcall;

var
   GetNetworkParams: TGetNetworkParams = nil;
   ConfigDNS1, ConfigDNS2 : String;
   List : TStringList;

implementation

{$R *.dfm}

uses Registry;

const
   sIPHLPAPI = 'iphlpapi.dll';

var
   hIPHLPAPI: HModule = 0;

function IsIpHelperAvailable: Boolean;
// Returns True if IP Helper API is available.
// Note: Even if available, some functions may fail with ERROR_NOT_SUPPORTED.
begin
   Result := ( hIPHLPAPI <> 0 );
end;

procedure Dialog_DNSFinder;
begin
   With TfrmDNSfinder.Create(NIL) do try
      ShowModal
   finally
      free
   end
end;

procedure TfrmDNSfinder.butOKClick(Sender: TObject);
begin
   ConfigDNS1 := ComboBox1.Text;
   ConfigDNS2 := ComboBox2.Text;
end;

function GetDNS(Index: Integer): String;
var  SL: TStringList;
begin
   Result := '127.0.0.1';
   SL := TStringList.Create;
   try
      if DNSFinder(SL) and (SL[Index]<>'') then Result := SL[Index];
   finally SL.Free end;
end;

function DNSFinder( const List: TStrings ): Boolean;

   procedure SrvAdd(const Srv: String);
   // add server to list, filter duplicates
   var  S: String;
   begin
      S := Trim( Srv );
      if length(S) = 0 then exit;
      if Assigned( List ) then begin
         if List.IndexOf(S) >= 0 then exit;
         List.Add(S);
      end;
      Result := True;
   end;

   procedure SrvListAdd(const SrvList: String);
   // add SP separated list of servers
   var  i: Integer;
        L, S: String;
   begin
      L := Trim( SrvList );
      while length( L ) > 0 do begin
         i := Pos( ' ', L );
         if i = 0 then begin
            S := L;
            L := '';
         end else begin
            S := copy( L, 1, i-1 );
            System.Delete( L, 1, i );
         end;
         SrvAdd(S);
      end;
   end;

const
   // default registry keys of TCP/IP settings on NT and 95 platforms
   RegTcpNT = 'SYSTEM\CurrentControlSet\Services\Tcpip\Parameters';
   RegTcp95 = 'SYSTEM\CurrentControlSet\Services\VxD\MSTCP';

var
   FixedInfo: PFIXED_INFO;
   BufLen   : ULONG;
   pIPAddr  : PIP_ADDR_STRING;
   L : String;

begin
   Result := False;
   if Assigned( List ) then List.Clear;

   // 1.) Read by "IP Helper API"
   //     (not available on all systems, so DLL is only loaded dynamically)
   if IsIpHelperAvailable then begin
      FixedInfo := PFIXED_INFO( GlobalAlloc( GPTR, sizeof(FIXED_INFO) ) );
      BufLen := sizeof( FIXED_INFO );
      if GetNetworkParams(FixedInfo, BufLen) = ERROR_BUFFER_OVERFLOW then begin
         GlobalFree( Cardinal(FixedInfo) );
         FixedInfo := PFIXED_INFO( GlobalAlloc( GPTR, BufLen ) );
      end;
      if GetNetworkParams(FixedInfo, BufLen) = ERROR_SUCCESS then begin
         SrvAdd(FixedInfo.DnsServerList.IpAddress.S);
         pIPAddr := FixedInfo.DnsServerList.Next;
         while Assigned( pIPAddr ) do begin
            SrvAdd(pIPAddr.IpAddress.S);
            pIPAddr := pIPAddr.Next;
         end;
      end;
      GlobalFree( Cardinal(FixedInfo) );
   end;

   // 2.) Read from dynamic RAS entries in registry
   //     (only available while connected)
   with TRegistry.Create( KEY_READ ) do try
      RootKey := HKEY_LOCAL_MACHINE;
      L := '';
      if OpenKey( RegTcpNT + '\Transient', False ) then begin
         L := ReadString( 'NameServer' );
      end else if OpenKey( RegTcp95 + '\Transient', False ) then begin
         L := ReadString( 'NameServer' );
      end;
      if L <> '' then SrvListAdd(L);
   finally Free end;

   // 3.) Read from TCP/IP settings in registry
   with TRegistry.Create( KEY_READ ) do try
      RootKey := HKEY_LOCAL_MACHINE;
      L := '';
      if OpenKey( RegTcpNT, False ) then begin
         L := ReadString( 'NameServer' );
         if L = '' then L := ReadString( 'DhcpNameServer' );
      end else if OpenKey( RegTcp95, False ) then begin
         L := ReadString( 'NameServer' );
         if L = '' then L := ReadString( 'DhcpNameServer' );
      end;
      if L <> '' then SrvListAdd(L);
   finally Free end;
end;

procedure TfrmDNSfinder.FormCreate(Sender: TObject);

begin
   ConfigDNS1 := CfgIni.ReadString ('Fr', 'DirectMX.DNS1', DirectMXDNS1);
   ConfigDNS2 := CfgIni.ReadString ('Fr', 'DirectMX.DNS2', DirectMXDNS2);
   if ConfigDNS1 <> '' then begin
      ComboBox1.Text := ConfigDNS1;
      ComboBox1.Items.Add(ConfigDNS1);
      ComboBox2.Items.Add(ConfigDNS1);
   end;
   if ConfigDNS2 <> '' then begin
      ComboBox2.Text := ConfigDNS2;
      ComboBox1.Items.Add(ConfigDNS2);
      ComboBox2.Items.Add(ConfigDNS2);
   end;
end;

procedure TfrmDNSfinder.butDetectClick(Sender: TObject);
var  SL: TStringList;
     i: integer;
begin
   lbDetected.Clear;
   ComboBox1.Clear;
   ComboBox2.Clear;
   FormCreate(nil);
   SL := TStringList.Create;
   try
      DNSFinder(SL);
      for i := 0 to SL.Count-1 do begin
         lbDetected.Items.Add(SL[i]);
         ComboBox1.Items.Add(SL[i]);
         ComboBox2.Items.Add(SL[i]);
      end;
   finally SL.Free end;
end;

procedure Load_IPHLPAPI;
begin
   hIPHLPAPI := LoadLibrary( sIPHLPAPI );
   if hIPHLPAPI <= HINSTANCE_ERROR then begin
      hIPHLPAPI := 0;
      exit;
   end;

   @GetNetworkParams := GetProcAddress( hIPHLPAPI, 'GetNetworkParams' );

   if not Assigned( GetNetworkParams ) then begin
      FreeLibrary( hIPHLPAPI );
      hIPHLPAPI := 0;
   end;
end;

initialization
   Load_IPHLPAPI;

finalization
   if hIPHLPAPI <> 0 then FreeLibrary( hIPHLPAPI );

end.
