unit zDirectMX;

interface

uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
   Dialogs, IdBaseComponent, IdComponent, IdUDPBase, IdUDPClient, IdDNSResolver,
   cLogfile, cStdForm, StdCtrls, Global;

type
  TDirectMX = class
   private
      fNSServers: TStringList;
      DNS: string;
      Domain: string;
   public
      fMailServers: TStringList;
      function QueryMXServers(MailOutFile: String): Boolean;
      function GetMXList: Integer;
      function SendQuery(Question: TQueryType): Integer;
      function GetNSServers(DirectMXDNS: string): Integer;
      function GetMailServers(QueryType: TQueryType): Boolean;
      procedure ResolveErrorNotify(ErrCode: integer; MailFile: String);
      procedure CheckMailServersList;
      constructor Create(ADomain: String);
      destructor Destroy; override;
   end;

function PadZero(s: String): String;

implementation

uses Main, Inifiles, Config, cMailRouter, uDateTime, uTools, cIPAccess, zDNSFinder;

constructor TDirectMX.Create(ADomain: String);
begin
   inherited Create;
   Domain := ADomain;
   fNSServers := TStringList.Create;
   fMailServers   := TStringList.Create;
end;

destructor TDirectMX.Destroy;
begin
   if Assigned(fNSServers) then fNSServers.Free;
   if Assigned(fMailServers) then fMailServers.Free;
   inherited Destroy;
end;

function TDirectMX.QueryMXServers(MailOutFile: String): Boolean;
var
   ListOK: Integer;
begin
   Domain := TrimWhSpace(Domain);
   if ((pos('[',Domain) = 1) and (pos(']',Domain) = length(Domain))) then
      Domain:=Copy(Domain,2,length(Domain)-2);
   if not IsValidIP(Domain) then begin
      ListOK := GetMXList;
      if ((ListOK > 0) and MXResolveErrorNotify) then begin
         ResolveErrorNotify(ListOK, MailOutFile);
         Result := False;
      end else Result := True;
   end else begin
      fMailServers.clear;
      fMailServers.Append(Domain);
      Result := True;
   end;
end;

function TDirectMX.GetMXList: Integer;
var
   NSQuery: Integer;
begin
   DirectMXDNS1 := CfgIni.ReadString ('Fr', 'DirectMX.DNS1', DirectMXDNS1);
   DirectMXDNS2 := CfgIni.ReadString ('Fr', 'DirectMX.DNS2', DirectMXDNS2);
   if Def_AutoDNS = true then begin
      DirectMXDNS1 := GetDNS(0);
      DirectMXDNS2 := GetDNS(1);
      Log( LOGID_INFO, TrGlF(kLog, 'DirectMX.GetDNS',
               'Searching DNS...', ' ') + DirectMXDNS1 + ' - ' + DirectMXDNS2 );
   end else begin
      if (DirectMXDNS1 = '') or (DirectMXDNS2 = '') then begin
         if DirectMXDNS1 = '' then DirectMXDNS1 := GetDNS(0);
         if DirectMXDNS2 = '' then DirectMXDNS2 := GetDNS(1);
         Log( LOGID_INFO, TrGlF(kLog, 'DirectMX.GetDNS',
               'Searching DNS...', ' ') + DirectMXDNS1 + ' - ' + DirectMXDNS2 );
      end;
   end;
   NSQuery := GetNSServers(DirectMXDNS1);
   if NSQuery > 0 then begin
      if NSQuery = 2 then Log( LOGID_DETAIL, TrGlF(kLog, 'DirectMX.NoDNS1Response',
         'No response from primary DNS server %s, trying secondary DNS.', DNS) );
      NSQuery := GetNSServers(DirectMXDNS2);
         if NSQuery > 0 then begin
            case NSQuery of
               1: begin
                     Log( LOGID_ERROR, TrGl(kLog, 'DirectMX.InvalidDomainName',
                        'Invalid domain name: ' ) + Domain );
                     result := NSQuery;
                     exit;
                  end;
               2: begin
                     Log( LOGID_ERROR, TrGl(kLog, 'DirectMX.NoDNS2Response',
                        'No response from primary and secondary DNS servers, aborting.') );
                     result := NSQuery;
                     exit;
                   end;
            end;
         end;
   end;
   if not GetMailServers([qtMX]) then begin
      Log( LOGID_DETAIL, TrGlF(kLog, 'DirectMX.NoMXRecord',
         'No MX record for domain %s, trying A record.', Domain) );
      if not GetMailServers([qtA]) then begin
         Log( LOGID_ERROR, TrGlF(kLog, 'DirectMX.NoARecord',
            'No MX nor A record for domain %s, aborting.', Domain) );
         result := 3;
      end else result := 0
   end else result := 0;
end;

function TDirectMX.SendQuery(Question: TQueryType): Integer;

begin
   Result := 0;
   with HamsterMainWindow.IdDNSResolver1 do begin
      QueryResult.Clear;
      Host := DNS;
      ReceiveTimeout := MXReceivedTimeout*1000;
      Log( LOGID_DEBUG, 'DNS : '+Host );
      try
         QueryRecords := Question;
         Resolve( Domain );
      except
         on E : Exception do begin
            if pos('DNS Server Reports Query Name Error', e.message) > 0 then begin
               Log( LOGID_DEBUG, TrGl(kLog, 'DirectMX.InvalidDomainName',
                  'Invalid domain name: ' ) + Domain );
               Result := 1;
            end else begin
               Log( LOGID_DEBUG, TrGl(kLog, 'DirectMX.ErrorResolvingDomain',
                  'Error resolving domain ' ) + Domain );
               Result := 2;
            end;
         end;
      end;
   end;
end;

function TDirectMX.GetNSServers(DirectMXDNS: string): Integer;
var
   i, Query : Integer;
   r: TResultRecord;

begin
   DNS := DirectMXDNS;
   fNSServers.clear;
   fNSServers.Sorted := true;
   fNSServers.Duplicates := dupIgnore;
	Query := SendQuery([qtStar]);
   if Query = 0 then begin
	   with HamsterMainWindow.IdDNSResolver1 do begin
	      if QueryResult.Count > 0 then begin
	         for i := 0 to QueryResult.Count - 1 do begin
	            r := QueryResult.Items[i];
               if r.RecType = qtNS then begin
                  fNSServers.Append(TNSRecord(r).HostName);
               end;
	         end;
            for i := 0 to fNSServers.count - 1 do begin
               Log( LOGID_DEBUG, TrGlF(kLog, 'DirectMX.AuthoritativeNS',
                  'Authoritative DNS #%s : %s', [IntToStr(i+1), fNSServers[i]]) );
            end;
	      end;
	   end;
	end;
   Result := Query;
end;

function TDirectMX.GetMailServers(QueryType: TQueryType): Boolean;
var
   i, x : integer;
   r: TResultRecord;

begin
   result := false;
   x := 0;
   fMailServers.Sorted := true;
   fMailServers.Duplicates := dupIgnore;
   while ((x < fNSServers.Count) and (result = false)) do begin
      fMailServers.clear;
      with HamsterMainWindow.IdDNSResolver1 do begin
         DNS := fNSServers[x];
         SendQuery(QueryType);
         if QueryType = [qtMX] then begin
            if QueryResult.Count > 0 then begin
               for i := 0 to QueryResult.Count - 1 do begin
   	            r := QueryResult.Items[i];
                  if r.RecType = qtMX then begin
                     fMailServers.Append(PadZero(IntToStr(TMXRecord(r).Preference))
                        + '=' + TMXRecord(r).ExchangeServer);
                  end;
               end;
               If fMailServers.Count > 0 then CheckMailServersList;
               Log( LOGID_DEBUG, 'MXs :' );
               for i := 0 to fMailServers.count - 1 do begin
                  Log( LOGID_DEBUG, fMailServers.Strings[i]);
               end;
               If fMailServers.Count > 0 then Result := true;
            end;
         end else begin
            if QueryResult.Count > 0 then begin
               for i := 0 to QueryResult.Count - 1 do begin
   	            r := QueryResult.Items[i];
                  if (r.RecType = qtA) and (r.Name = Domain) then begin
                     fMailServers.Append(TARecord(r).IPAddress);
                  end;
               end;
               If fMailServers.Count > 0 then CheckMailServersList;
               Log( LOGID_DEBUG, 'MXs :' );
               for i := 0 to fMailServers.count - 1 do begin
                  Log( LOGID_DEBUG, fMailServers.Strings[i]);
               end;
               If fMailServers.Count > 0 then Result := true;
            end;
         end;
      end;
      inc(x);
   end;
end;

function PadZero(s: String): String;
begin
   if length(s) < 2 then s := '0' + s;
   if length(s) < 3 then s := '0' + s;
   Result := s;
end;

procedure TDirectMX.ResolveErrorNotify(ErrCode: integer; MailFile: String);
var
   i, k, Attempts: integer;
   s, MailFrom ,MailPath, MailName, ResLog: String;
   TheMail: TStringlist;

begin
   case ErrCode of
      1: ResLog:='995 Invalid domain name.';
      2: ResLog:='996 No response from DNS servers.';
      3: ResLog:='997 No MX nor A record for this domain.';
   end;

   {JW} {Notify}
   MailPath := ExtractFilePath( MailFile );
   MailName := ExtractFilename( MailFile );
   i := Pos( '.' + CfgIni.ReadString( 'Setup', 'mail.ext.out', 'msg' ), lowercase(MailName) );
   if i>0 then MailName:=copy(MailName,1,i-1);
   i := Pos( '-', MailName );
   if i=0 then begin
      Attempts := 0;
   end else begin
      Attempts := strtoint( copy(MailName,i+1,3) );
      MailName := copy( MailName, 1, i-1 );
   end;
   // notify sender of errors

   {/JW}
   TheMail := TStringlist.Create;
   try
	   try
	      TheMail.LoadFromFile( MailFile )
	   except
	      Log( LOGID_Error, TrGl(kLog, 'SMTP.SendMail.CantLoad',
	         '[DirectMX, SMTP] can not load Mailfile ')+ ExtractFilename(MailFile));
	      Exit;
	   end;
	
	   s := '[' + OUR_VERINFO + ']' + #13#10#13#10
	   + TrGl(kLog, 'SMTP.SendMail.DeliveryFailed1',
         'The following mail could not be delivered to any or all recipients:')
      + #13#10#13#10
	   + '> ' + MailFile + #13#10#13#10
	   + TrGl(kLog, 'SMTP.SendMail.DeliveryFailed2',
         'Delivery-results:') + #13#10#13#10
	   + '| ' + ResLog + #13#10#13#10
	   + TrGl(kLog, 'SMTP.SendMail.DeliveryFailed3',
         'Header-lines of undelivered mail:') + #13#10#13#10;
	   k := TheMail.IndexOf('');
	   If k < 0 then k := TheMail.Count-1;
	   for i:=0 to k-1 do begin
	      s := s + '| ' + TheMail[i] + #13#10;
	   end;
	   {JW} {InfoMail}
      MailFrom := TrimWhSpace(copy(TheMail[0],12,99));
	   if pos('<',Mailfrom) = 1 then Mailfrom:=Copy(MailFrom,2,MaxInt);
	   if pos('>',Mailfrom) = length(MailFrom) then
	      Mailfrom := Copy(MailFrom,1,length(MailFrom)-1);
	   {/JW}
	   SendInfoMail( MailFrom, TrGl(kLog, 'SMTP.SendMail.DeliveryFailed4',
         '[' + OUR_VERINFO + '] Mail-delivery failed!'), s );
	
	   inc( Attempts );
	   if Attempts>=Def_SendMailAttemptsMax then begin
	      ResLog := '999 Delivery failed; no further attempts.';
	      if Def_SendMailAttemptsDel then begin
	         DeleteFile( MailFile );
	      end else begin
	         // rename file to prevent further attempts
	         MailName := MailName + '-' + inttostr(Attempts-1);
	         RenameFile( MailFile, MailPath + MailName + '.err' );
	         // notify admin of orphaned file
	         s := '[' + OUR_VERINFO + ']' + #13#10#13#10
	            + TrGlF(kLog, 'SMTP.SendMail.DeliveryAborted1',
                  'After %s attempts, an undeliverable mail was renamed to:',
                  inttostr(Attempts)) + #13#10#13#10
	            + '> ' + MailPath + MailName + '.err' + #13#10#13#10
	            + TrGl(kLog, 'SMTP.SendMail.DeliveryAborted2',
                  'This file will NOT be deleted or sent out again by Hamster-Fr.') + #13#10;
	         SendInfoMail( Def_Postmaster, TrGl(kLog, 'SMTP.SendMail.DeliveryAborted3',
               '[' + OUR_VERINFO + '] Mail-delivery aborted!'), s );
	      end;
	   end else begin
	      // mark file with new attempts-count
	      MailName := MailName + '-' + inttostr(Attempts);
	      RenameFile( MailFile, MailPath + MailName + '.' +
	         CfgIni.ReadString( 'Setup', 'mail.ext.out', 'msg' ) );
	   end;
   finally
      TheMail.Free
   end;

   // Note result in mailout.log:
   s := DateTimeToTimeStamp( Now ) + ' '
      + 'File='   + ExtractFilename( MailFile ) + ' '
      + 'Server=NA '
      + 'Result=' + ResLog;
   HamFileAppendLine ( PATH_LOGS + 'MailOut.log', s );
end;

procedure TDirectMX.CheckMailServersList;
var
   i, j, Rnd, Rnd1 : Integer;
   s, s1 : String;
   SamePref : Array of Integer;

begin
   Randomize;
   SetLength( SamePref, fMailServers.Count - 1 );
   fMailServers.Sorted := False;
   try

     // Si MX = FQDN local alors suppression des serveur de prfrence identique et suprieures
     for i := 0 to fMailServers.count - 1 do begin
         if copy(fMailServers.Strings[i], 5, 999) = Def_FQDN then begin
            s := copy(fMailServers.Strings[i], 1, 3);
            while i < fMailServers.count do fMailServers.Delete(i);
            j := i - 1;
            while j >= 0 do
               if copy(fMailServers.Strings[j], 1, 3) = s then begin
                  fMailServers.Delete(j);
                  dec(j);
               end else break;
            break;
         end;
      end;

      // Tri alatoire des MX de mme prfrence
      for i := 0 to fMailServers.count - 2 do begin
         s := copy(fMailServers.Strings[i], 1, 3);
         s1 := copy(fMailServers.Strings[i+1], 1, 3);
         if s = s1 then
            if i = 0 then SamePref[i] := 1 else SamePref[i] := SamePref[i-1] + 1;
      end;
      for i := fMailServers.count - 2 downto 0 do begin
         if SamePref[i] > 0 then begin
            Rnd := Random( SamePref[i]+1 )+i-SamePref[i]+1;
            Rnd1 := Random( SamePref[i]+1 )+i-SamePref[i]+1;
            fMailServers.Exchange(Rnd, Rnd1);
         end;
      end;
   finally
      SamePref := nil
   end;
end;
end.
